/*
 Copyright (c) 2010 Myles Metzer

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef TVOUT_H
#define TVOUT_H

/*
 This library provides a simple method for outputting data to a tv
 from a frame buffer stored in sram.  This implementation is done
 completly by interupt and will return give as much cpu time to the
 application as possible.
 
 Limitations:
  -	Currently only works with NTSC. PAl support would require modifying
	the timings in video_gen.h as well as the OCR1A ISR to allow for a 16bit
	line counter.
  -	each frame only consists of 256 scanlines vs 262 for the fake progressive
	signal this is supposed to generate (works fine on my tv).
  - virtical sync does not match the specs at all.
 

 current hardware setup:
 Pin9: Sync line:  -->|--/\/\/\--o
                   Diode 1Kohm   |
 Pin8: Video line: -->|--/\/\/\--o---------------- RCA tip
                   Diode 330ohm  |
                                 o--/--/\/\/\--o-- RCA GND
							     Switch  75ohm |
									           V
											  GND
											   
 
*/

// macros for readability when selecting mode.
#define _PAL					1
#define _NTSC					0

/*
 TVout.cpp contains a brief expenation of each function.
*/
class TVout {
	public:
		void start_render(unsigned char mode);
		void pause_render();
		void resume_render();
		void clear_screen();
		unsigned char horz_res();
		unsigned char vert_res();
		char char_line();
		void delay_frame(unsigned int x);
		void set_pixel(unsigned char x, unsigned char y, char c);
		
		unsigned char get_pixel(unsigned char x, unsigned char y,  unsigned char c);
		void draw_line(unsigned char x0, unsigned char y0, 
					   unsigned char x1, unsigned char y1, char c);
		void print_char(unsigned char x, unsigned char y, char c);
		void print_str(unsigned char x, unsigned char y, char *str);
		
		void draw_progressbar(unsigned char x0, unsigned char y0, unsigned char x1, unsigned char y1, char c, char d, char e);
		void draw_box(unsigned char x0, unsigned char y0, unsigned char x1, unsigned char y1, char c, char d, char e);
		
		void bres_line(int x1, int y1, int x2, int y2, unsigned char val);
		
		//some functions i added for another project (gijs gieskes)
		void scroll(unsigned char x0, unsigned char y0,
					unsigned char x1, unsigned char y1, char c);
		void invert();
		void fade(char v, char c);
		void shift(unsigned char c, char speed);
		void noise(unsigned char v);
		void set_byte(unsigned int i, unsigned char c);
		
		//this function is for adjusting the virtical scroll
		void scan_line(unsigned int v);
		
		void flood_fill(int x, int y, unsigned char c);
		void draw_line_limited(unsigned char x0, unsigned char y0,
					  unsigned char x1, unsigned char y1, char c);
		
};

static void inline sp(unsigned char x, unsigned char y, char c);
static unsigned char inline gp(unsigned char x, unsigned char y);

#endif
