/*
 Copyright (c) 2010 Myles Metzer

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
*/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>

#include "video_gen.h"
#include "video_properties.h"

//gobal variables 
unsigned int scan_lines;
volatile int g_scanLine;
unsigned char g_screen[_RESOLUTION_HORZ_BYTES*_RESOLUTION_VERTICAL];
volatile unsigned char g_status;

unsigned int renderLine;	//offset from screen of the current line

// render a line
ISR(TIMER1_COMPB_vect) {
	//PORTB &= ~_RENDER;
	//PORTB &= ~_RENDER;
		switch (g_scanLine) {
			case _PAL_LINE_START_VSYNC:			//start vsync	(this is 0)
				OCR1A = _PAL_CYCLES_VIRT_SYNC;
				break;
			case _PAL_LINE_STOP_VSYNC:			//stop vsync	(this is 7)
				OCR1A = _PAL_CYCLES_HORZ_SYNC;
				break;
			
			case _PAL_LINE_START_RENDER:		//start to render each line (this is 92.5)
				g_status |= _RENDER;
				renderLine = 0;
				break;
			case _PAL_LINE_STOP_RENDER:			//stop rendering each line  (thsi is 201.5)
				g_status &= ~_RENDER;
				break;
		}
		if (_IS_RENDER) {
			wait_until(_PAL_CYCLES_OUTPUT_START);
			render_line();
			g_scanLine++;
		}
		else if (g_scanLine == scan_lines)//_PAL_LINE_FRAME)//
			g_scanLine = 0;
		else
			g_scanLine++;
	
}



/* render a 1bit black and white scanline
*/
void render_line() {

	__asm__ __volatile__ (
		"ADD	r26,r28\n\t"
		"ADC	r27,r29\n\t"
		//save PORTB
		"IN		r16,%[port]\n\t"
		"ANDI	r16,0xFD\n\t"
	
		//output thingy
		"LD		__tmp_reg__,X+\n\t"		//1
		"byteshift %[port]\n\t"
#if (_RESOLUTION_HORZ_BYTES >= 2)
		"LD		__tmp_reg__,X+\n\t"		//2
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 3)
		"LD		__tmp_reg__,X+\n\t"		//3
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 4)
		"LD		__tmp_reg__,X+\n\t"		//4
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 5)
		"LD		__tmp_reg__,X+\n\t"		//5
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 6)
		"LD		__tmp_reg__,X+\n\t"		//6
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 7)
		"LD		__tmp_reg__,X+\n\t"		//7
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 8)
		"LD		__tmp_reg__,X+\n\t"		//8
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 9)
		"LD		__tmp_reg__,X+\n\t"		//9
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 10)
		"LD		__tmp_reg__,X+\n\t"		//10
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 11)
		"LD		__tmp_reg__,X+\n\t"		//11
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 12)
		"LD		__tmp_reg__,X+\n\t"		//12
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 13)
		"LD		__tmp_reg__,X+\n\t"		//13
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 14)
		"LD		__tmp_reg__,X+\n\t"		//14
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 15)
		"LD		__tmp_reg__,X+\n\t"		//15
		"byteshift %[port]\n\t"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 16)
		"LD		__tmp_reg__,X+\n\t"		//16
		"byteshift %[port]\n"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 17)
		"LD		__tmp_reg__,X+\n\t"		//17
		"byteshift %[port]\n"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 18)
		"LD		__tmp_reg__,X+\n\t"		//18
		"byteshift %[port]\n"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 19)
		"LD		__tmp_reg__,X+\n\t"		//19
		"byteshift %[port]\n"
#endif
#if (_RESOLUTION_HORZ_BYTES >= 20)
		"LD		__tmp_reg__,X+\n\t"		//20
		"byteshift %[port]\n"
#endif
	
	"EndLine_%=:\n\t"
		"ANDI	r27,0\n\t"				// r27 is XH
		"BST	r27,0\n\t"
		"output %[port]\n\t"
	// assembly variable IO:
	:
	: [port] "i" (_SFR_IO_ADDR(PORTB)),
	  "x" (g_screen),
	  "y" (renderLine)
	: "r16" // try to remove this clobber later...
	);
#if (_RESOLUTION_VSCALE == 2)
	if (g_scanLine & 0x1)
		renderLine += _RESOLUTION_HORZ_BYTES;
#else
	renderLine += _RESOLUTION_HORZ_BYTES;
#endif
}

static void inline wait_until(unsigned char time) {
	__asm__ __volatile__ (
			"sub	%[time], %[tcnt1l]\n\t"
			"subi	%[time], 10\n"
		"100:\n\t"
			"subi	%[time], 3\n\t"
			"brcc	100b\n\t"
			"subi	%[time], 0-3\n\t"
			"breq	101f\n\t"
			"dec	%[time]\n\t"
			"breq	102f\n\t"
			"rjmp	102f\n"
		"101:\n\t"
			"nop\n" 
		"102:\n"
		:
		: [time] "a" (time),
		[tcnt1l] "a" (TCNT1L)
	);
}

// asm macros
__asm__ __volatile__ (
	// macro for the delay between regular pixels(not the first pixel)
	".macro pixeldelay\n\t"				// delay time for setting pixel width
#if (_CYCLES_PIXEL >= 6)
		"NOP\n\t"						// one can be removed for smaller
#endif
#if (_CYCLES_PIXEL >= 7)
		"NOP\n\t"
#endif
#if (_CYCLES_PIXEL >= 8)
		"NOP\n\t"
#endif
#if (_CYCLES_PIXEL >= 9)
		"NOP\n\t"
#endif
#if (_CYCLES_PIXEL >= 10)
		"NOP\n\t"
#endif
		"NOP\n\t"						// pixels dont forget to remove the
		"NOP\n\t"						// nop at the start of byteshift
	".endm\n"
	
	//macro that defines the actual output code
	#if defined(__AVR_ATmega1280__)
	".macro output port\n\t"
		"BLD	r16,6\n\t"				//output pin of the port here pinB0
		"OUT	\\port,r16\n"			//and on the last line of asm
	".endm\n"
#else
	// output for ATMega8,ATMega168,ATMega328
	".macro output port\n\t"
		"BLD	r16,0\n\t"				//output pin of the port here pinB0
		"OUT	\\port,r16\n"			//and on the last line of asm
	".endm\n"
#endif

	".macro byteshift p\n\t"
#if (_CYCLES_PIXEL >= 6)
		"NOP\n\t"
#endif
#if (_CYCLES_PIXEL >= 7)
		"NOP\n\t"
#endif
#if (_CYCLES_PIXEL >= 8)
		"NOP\n\t"
#endif
#if (_CYCLES_PIXEL >= 9)
		"NOP\n\t"
#endif
#if (_CYCLES_PIXEL >= 10)
		"NOP\n\t"
#endif
		"BST	__tmp_reg__,7\n\t"
		"output \\p\n\t"
		"pixeldelay\n\t"
		"BST	__tmp_reg__,6\n\t"
		"output \\p\n\t"
		"pixeldelay\n\t"
		"BST	__tmp_reg__,5\n\t"
		"output \\p\n\t"
		"pixeldelay\n\t"
		"BST	__tmp_reg__,4\n\t"
		"output \\p\n\t"
		"pixeldelay\n\t"
		"BST	__tmp_reg__,3\n\t"
		"output \\p\n\t"
		"pixeldelay\n\t"
		"BST	__tmp_reg__,2\n\t"
		"output \\p\n\t"
		"pixeldelay\n\t"
		"BST	__tmp_reg__,1\n\t"
		"output \\p\n\t"
		"pixeldelay\n\t"
		"BST	__tmp_reg__,0\n\t"
		"output \\p\n"
	".endm\n\t"
);
