/*
 Copyright (c) 2010 Myles Metzer

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
*/

/*
 This File contains the timing definitions for the TVout AVR composite video
 generation Library
*/
#ifndef VIDEO_TIMING_H
#define	VIDEO_TIMING_H

//screen properties
#define _RESOLUTION_HORIZONTAL 	128 	// must be a multiple of 8!!!!
#define _RESOLUTION_VERTICAL 	104//110		//VS08 96

// automatically select the virtical scale factor.
#if ( _RESOLUTION_VERTICAL  <= 120)
#define _RESOLUTION_VSCALE 		2
#else
#define _RESOLUTION_VSCALE 		1
#endif

// automatically select the horizonal output speed.
#define _RESOLUTION_HORZ_BYTES	(_RESOLUTION_HORIZONTAL / 8)
#if (_RESOLUTION_HORIZONTAL > 128)
#define _CYCLES_PIXEL			5
#elif (_RESOLUTION_HORIZONTAL > 104)
#define _CYCLES_PIXEL			6
#elif (_RESOLUTION_HORIZONTAL > 96)
#define _CYCLES_PIXEL			7
#elif (_RESOLUTION_HORIZONTAL > 80)
#define _CYCLES_PIXEL			9
#elif (_RESOLUTION_HORIZONTAL > 72)
#define _CYCLES_PIXEL			10
#endif

#define _CYCLES_PER_US			(F_CPU / 1000000)

//Timing settings for NTSC
#define _NTSC_TIME_SCANLINE			63.625
#define _NTSC_TIME_HORZ_SYNC		4.7
#define _NTSC_TIME_OUTPUT_START		11
#define _NTSC_TIME_VIRT_SYNC		(_NTSC_TIME_SCANLINE - _NTSC_TIME_HORZ_SYNC)

#define _NTSC_LINE_FRAME			262
#define _NTSC_LINE_START_VSYNC		0
#define _NTSC_LINE_STOP_VSYNC		3
#if ((((_NTSC_LINE_FRAME/2) - (_RESOLUTION_VERTICAL/2*_RESOLUTION_VSCALE)) + _NTSC_LINE_STOP_VSYNC) % 2)
#define _NTSC_LINE_START_RENDER		(((_NTSC_LINE_FRAME/2) - (_RESOLUTION_VERTICAL/2*_RESOLUTION_VSCALE)) + _NTSC_LINE_STOP_VSYNC+1)
#else
#define _NTSC_LINE_START_RENDER		(((_NTSC_LINE_FRAME/2) - (_RESOLUTION_VERTICAL/2*_RESOLUTION_VSCALE)) + _NTSC_LINE_STOP_VSYNC)
#endif

#define _NTSC_LINE_STOP_RENDER		((_NTSC_LINE_START_RENDER + (_RESOLUTION_VERTICAL * _RESOLUTION_VSCALE)) - 1)

#define _NTSC_CYCLES_SCANLINE		((_NTSC_TIME_SCANLINE * _CYCLES_PER_US) - 1)
#define _NTSC_CYCLES_HORZ_SYNC		((_NTSC_TIME_HORZ_SYNC * _CYCLES_PER_US) - 1)
#define _NTSC_CYCLES_OUTPUT_START	((_NTSC_TIME_OUTPUT_START * _CYCLES_PER_US) - 1)
#define _NTSC_CYCLES_VIRT_SYNC		((_NTSC_TIME_VIRT_SYNC * _CYCLES_PER_US) - 1)

//Timing settings for PAL
#define _PAL_TIME_SCANLINE			64	   //aangepast, was 64, crystal fine tuning.
#define _PAL_TIME_HORZ_SYNC			4.7	
#define _PAL_TIME_OUTPUT_START		11.75
#define _PAL_TIME_VIRT_SYNC			(_PAL_TIME_SCANLINE - _PAL_TIME_HORZ_SYNC)

#define _PAL_LINE_FRAME				311 //aangepast, was 308, verticaal scroll (TBF)
#define _PAL_LINE_START_VSYNC		0
#define _PAL_LINE_STOP_VSYNC		7
#if ((((_PAL_LINE_FRAME/2) - (_RESOLUTION_VERTICAL/2*_RESOLUTION_VSCALE)) + _PAL_LINE_STOP_VSYNC) % 2)
#define _PAL_LINE_START_RENDER		(((_PAL_LINE_FRAME/2) - (_RESOLUTION_VERTICAL/2*_RESOLUTION_VSCALE)) + _PAL_LINE_STOP_VSYNC+1)
#else
#define _PAL_LINE_START_RENDER		(((_PAL_LINE_FRAME/2) - (_RESOLUTION_VERTICAL/2*_RESOLUTION_VSCALE)) + _PAL_LINE_STOP_VSYNC)
#endif
#define _PAL_LINE_STOP_RENDER		((_PAL_LINE_START_RENDER + (_RESOLUTION_VERTICAL * _RESOLUTION_VSCALE)) - 1)

#define _PAL_CYCLES_SCANLINE		((_PAL_TIME_SCANLINE * _CYCLES_PER_US) - 1)
#define _PAL_CYCLES_HORZ_SYNC		((_PAL_TIME_HORZ_SYNC * _CYCLES_PER_US) - 1)
#define _PAL_CYCLES_OUTPUT_START	((_PAL_TIME_OUTPUT_START * _CYCLES_PER_US) - 1)
#define _PAL_CYCLES_VIRT_SYNC		((_PAL_TIME_VIRT_SYNC * _CYCLES_PER_US) - 1)

#endif